import genanki
import pandas as pd
from typing import List
from functools import reduce

from ankimaker.config import Config, FilterConfig
from ankimaker import generator, config


def load_csv(path):
    df = pd.read_csv(path, header=Config.header, sep=Config.separators)
    df_columns_are_unnamed = all(map(lambda x: str(x).isnumeric(), df.columns))
    if df_columns_are_unnamed:
        Config.answer_column = int(Config.answer_column)
        Config.question_column = int(Config.question_column)
    df = apply_filters(df)
    return df


def add_df_to_deck(df: pd.DataFrame, deck: genanki.Deck):
    model = generator.create_model()

    for entry in df.to_dict('records'):
        question = entry[Config.question_column]
        answer = entry[Config.answer_column]
        content_fields = (question, answer)
        note = generator.create_note(model, fields=content_fields)
        deck.add_note(note)
    return deck


def handle_config(config_file_path):
    if config_file_path is None:
        Config.header = None
        Config.question_column = 0
        Config.answer_column = 1
    else:
        config.load_config_file(config_file_path)


def apply_filters(df: pd.DataFrame) -> pd.DataFrame:
    """

    :param df:
    :return:
    """
    there_are_no_filter_to_apply = len(Config.filters) == 0
    if there_are_no_filter_to_apply:
        return df
    is_in_configured_filter_rules = load_filter_from_config(df)
    df_filtered = df[is_in_configured_filter_rules]
    return df_filtered


def load_filter_from_config(df: pd.DataFrame) -> pd.Series:

    group_filters: List[pd.Series] = list()
    for group in Config.filters:
        if len(group) > 0:
            group_filters.append(
                create_group_filter(df, group)
            )
    config_filter = reduce(lambda a, b: a | b, group_filters)
    return config_filter


def create_group_filter(df: pd.DataFrame, group: List[FilterConfig]) -> pd.Series:
    """

    :param df:
    :param group:
    :return:
    """
    rule: FilterConfig
    query: List[pd.Series] = list()
    for rule in group:
        assert_rule_is_valid(df, rule)
        is_in_rule = df[rule.column].apply(lambda x: x in rule.values)
        query.append(is_in_rule)
    is_in_group = reduce(lambda a, b: a & b, query)
    return is_in_group


def assert_rule_is_valid(df: pd.DataFrame, rule: FilterConfig):
    assert rule.column in df.columns


def basic_pandas_to_anki(csv_path, output_path, name, config_file_path):
    handle_config(config_file_path)
    df = load_csv(csv_path)
    deck = generator.deck.create_deck(name)
    add_df_to_deck(df, deck)
    generator.deck.save_deck(deck, output_path)
